--  PSL - HDL interface.
--  Copyright (C) 2002-2016 Tristan Gingold
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 2 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program.  If not, see <gnu.org/licenses>.

with Types; use Types;
with PSL.Nodes; use PSL.Nodes;

package PSL.Hash is
   --  Initialize the package.
   procedure Init;

   --  Get the PSL node for node HDL.
   --  Only one PSL node is created for an HDL node.
   function Get_PSL_Node (Hdl : Int32; Loc : Location_Type) return Node;
end PSL.Hash;
