entity tb_repro_sgn is
end tb_repro_sgn;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

architecture behav of tb_repro_sgn is
  signal clk : std_logic;
  signal a : signed(7 downto 0);
  signal b : signed(7 downto 0);
begin
  dut: entity work.repro_sgn
    port map (
      clk => clk, a => a, b => b);

  process
    procedure pulse is
    begin
      clk <= '0';
      wait for 1 ns;
      clk <= '1';
      wait for 1 ns;
    end pulse;
  begin
    a <= x"ab";
    pulse;
    assert b = x"ab" severity failure;

    a <= x"12";
    pulse;
    assert b = x"12" severity failure;
    wait;
  end process;
end behav;
